

const zipCache = new Map();
function loadZipFile() {
    let remoteUrl = "http://192.168.0.169:8001/bundle/testBundle.b4991.zip";
    cc.assetManager.loadAny({ url: remoteUrl, ext: '.bin' }, async (error, assets) => {
        if (error) return;
        let jszip = new JSZip();
        let zipData = await jszip.loadAsync(assets);
        let keys = Object.keys(zipData.files);
        for (let i = 0; i < keys.length; i++) {
            let key = keys[i];
            let data = zipData.files[key];
            let extName = key.split('.').pop();
            let tempImageName = key.split('/').pop();
            let asyncType = "";
            switch (extName) {
                case 'png':
                case 'jpg':
                case 'bmp':
                case 'jpeg':
                case 'gif':
                case 'ico':
                case 'tiff':
                case 'webp':
                case 'image':
                    asyncType = cc.sys.capabilities.imageBitmap && cc.macro.ALLOW_IMAGE_BITMAP ? 'blob' : 'base64';
                    data.async(asyncType)
                        .then((result) => {
                            zipCache.set(tempImageName, result);
                        });
                    break;
                case 'js':
                case 'json':
                    asyncType = 'string';
                    data.async(asyncType).then((result) => {
                        zipCache.set(tempImageName, result);
                    });
                    break;
                case 'mp3':
                case 'ttf':
                    asyncType = 'blob'; // 同样使用blob格式
                    data.async(asyncType)
                        .then((result) => {
                            zipCache.set(tempImageName, result);
                        });
                    break;
                case 'default':
                    break;
            }
        }
    });
}
loadZipFile()

function orginalLoader(url, options, onComplete) {
    const allowImageBitmap = cc.sys.capabilities.imageBitmap && cc.macro.ALLOW_IMAGE_BITMAP;
    if (allowImageBitmap) {
        options.responseType = 'blob';
        cc.assetManager.downloader.downloadFile(url, options, (error, data) => {
            if (error) {
                onComplete && onComplete(null, allowImageBitmap ? "" : new Image());
            } else {
                onComplete && onComplete(null, data);
            }
        });
    } else {
        cc.assetManager.downloader.downloadDomImage(url, options, (error, data) => {
            if (error) {
                onComplete && onComplete(null, allowImageBitmap ? "" : new Image());
            } else {
                onComplete && onComplete(null, data);
            }
        });
    }
}

let downloadImage = function (url, options, onComplite) {
    let fileName = cc.path.basename(url);
    if (zipCache.has(fileName)) {
        console.log('load file from zipFile:', fileName);
        onComplite && onComplite(null, zipCache.get(fileName));
    } else {
        orginalLoader(url, options, onComplite);
    }
}

// 动态执行 JavaScript 代码
function executeJSFile(code) {
    const script = document.createElement('script');
    script.textContent = code;
    document.body.appendChild(script);
}

const REGEX = /^(?:\w+:\/\/|\.+\/).+/;
let downloadBundle = function (nameOrUrl, options, onComplete) {
    let bundleName = cc.path.basename(nameOrUrl);
    let url = nameOrUrl;
    if (!REGEX.test(url)) url = 'assets/' + bundleName;
    var version = options.version || cc.assetManager.downloader.bundleVers[bundleName];
    var count = 0;
    // var config = `${url}/config.${version ? version + '.' : ''}json`;
    var config = `config.${version ? version + '.' : ''}json`;
    let out = null, error = null;
    if (zipCache.has(config)) {
        url = '';
        count++;
        config = `config.${version ? version + '.' : ''}json`;
        out = JSON.parse(zipCache.get(config));
        out.base = "";
        if (count === 2) {
            onComplete(null, out);
        }
    } else {
        url = 'assets/' + bundleName;
        config = `${url}/config.${version ? version + '.' : ''}json`;
        cc.assetManager.downloader._downloaders['.json'](config, options, function (err, response) {
            if (err) {
                error = err;
            }
            out = response;
            out && (out.base = url + '/');
            count++;
            if (count === 2) {
                onComplete(error, out);
            }
        });
    }

    var js = `index.${version ? version + '.' : ''}js`;
    if (zipCache.has(js)) {
        count++;
        executeJSFile(zipCache.get(js));
        if (count === 2) {
            onComplete(null, out);
        }
    } else {
        url = 'assets/' + bundleName;
        js = `${url}/index.${version ? version + '.' : ''}js`;
        cc.assetManager.downloader._downloaders['.js'](js, options, function (err) {
            if (err) {
                error = err;
            }
            count++;
            if (count === 2) {
                onComplete(error, out);
            }
        });
    }

}

var zipCacheUrl = "";
let injectXMLHttpRequest = function () {
    // const originalSrcDescriptor = Object.getOwnPropertyDescriptor(HTMLScriptElement.prototype, 'src');

    // Object.defineProperty(HTMLScriptElement.prototype, 'src', {
    //     get() {
    //         return originalSrcDescriptor?.get?.call(this);
    //     },
    //     set(value) {
    //         // 检查缓存
    //         if (zipCache.has(value)) {
    //             console.error('[zipCache.js]', value);
    //             const scriptContent = zipCache.get(value)
    //             const blob = new Blob([scriptContent.async('arraybuffer')], { type: 'application/javascript' });
    //             const blobUrl = URL.createObjectURL(blob);
    //             // 延迟设置 Blob URL
    //             setTimeout(() => {
    //                 originalSrcDescriptor?.set?.call(this, blobUrl);
    //             }, 50);

    //             // 清理内存
    //             this.addEventListener('load', () => URL.revokeObjectURL(blobUrl));
    //             return;
    //         }
    //         // 无缓存则正常加载
    //         originalSrcDescriptor?.set?.call(this, value);
    //     },
    //     configurable: true,
    // });


    const accessor = Object.getOwnPropertyDescriptor(XMLHttpRequest.prototype, 'response');
    if (accessor === undefined)
        return false;

    // 注入新 open 方法
    const oldOpen = (XMLHttpRequest.prototype).open;
    // @ts-ignore
    (XMLHttpRequest.prototype).open = function (method, url, async, user, password) {
        const extension = url.split('.').pop();

        // 檢查zip 加載時是否有這個url
        if (zipCache.has(cc.path.basename(url))) {
            this.zipCacheUrl = cc.path.basename(url);
        } else {
            this.zipCacheUrl = null;
        }

        return oldOpen.apply(this, [method, url, async, user, password]);
    }

    // 注入新 send 方法
    const oldSend = (XMLHttpRequest.prototype).send;
    (XMLHttpRequest.prototype).send = async function (data) {
        if (this.zipCacheUrl) {
            // if (!resCache.has(this.zipCacheUrl)) {
            //     const cache = zipCache.get(this.zipCacheUrl);
            //     if (cache) {
            //         if (this.responseType === "json") {
            //             const text = await cache.async("text");
            //             resCache.set(this.zipCacheUrl, text);
            //         } else {
            //             const res = await cache.async(this.responseType);
            //             resCache.set(this.zipCacheUrl, res);
            //         }
            //         // zipCache.delete(this.zipCacheUrl);
            //     }
            // }

            if (typeof this.onload === "function") {
                const event = new ProgressEvent('load', {
                    lengthComputable: false,
                    loaded: 0,
                    total: 0
                });
                this.onload(event);
            }
            return;
        }
        return oldSend.apply(this, [data]);
    }

    Object.defineProperty((XMLHttpRequest.prototype), 'response', {
        get: function () {
            if (this.zipCacheUrl) {
                const res = zipCache.get(this.zipCacheUrl);
                return this.responseType === "json" ? JSON.parse(res) : res;
            }
            return accessor.get ? accessor.get.call(this) : undefined;
        },
        set: function () { },
        configurable: true
    });

    return true;
}
injectXMLHttpRequest();
cc.assetManager.downloader.register({
    ".png": downloadImage, ".jpg": downloadImage,
    ".webp": downloadImage,
    'bundle': downloadBundle
});
